#[forbid(unsafe_code)]
fn main() {
    // Let's swap two items in a `Vec`.
    let mut vec = Vec::from([1985, 2025]);

    // First try: grab mutable references to the items, then swap them.
    let first = vec.get_mut(0).unwrap();
    let second = vec.get_mut(1).unwrap();
    std::mem::swap(first, second);

    // // Second try: use a standard library function
    // vec.swap(0, 1);

    // Much safe! Very concurrency!
    println!("OUTPUT: {:?}", &vec);
}

// Claim 1: Creating two mutable references to (different) places
// within the memory owned by the same object is only possible
// through unsafe Rust.
//
// Claim 2: #[forbid(unsafe_code)] guarantees that no unsafe
// code is ever used.
//
// Claim 3: I need two mutable references to places in the
// memory owned by a `Vec` in order to swap them.
//
//
// Pick two.