// Example by Jack O'Connor
// https://www.youtube.com/watch?v=DG-VLezRkYQ

pub fn main() {
    rust_like_its_1985();
}

pub fn rust_like_its_1985() {
    let mut n = 0;

    // Taking a raw pointer is safe!
    let n_ptr = &raw mut n;
    
    // Dereferencing a pointer is unsafe, but not necessarily UB.
    let n_mut1 = unsafe { &mut *n_ptr };

    // UB: Multiple mutable references to the same place
    let n_mut2 = unsafe { &mut *n_ptr };
    
    // Or is it? I can just remember not to operate on them at the same time.
    *n_mut1 = 42;

    // Right?
    *n_mut2 = 0;

    // "It works for me"
    println!("OUTPUT: {}", *n_mut1);
}

pub fn rust_like_its_2025() {
    let mut n = 0;
    let n_ptr = &raw mut n;
    let n_mut1 = unsafe { &mut *n_ptr };
    let n_mut2 = unsafe { &mut *n_ptr };

    // So far this is the same. Let's move the rest to a function.
    println!("OUTPUT: {}", foo(n_mut1, n_mut2));

    // This works if you compile on debug.
    // It also works if `foo` is hinted as never inlined.
    // But it won't work anymore if non-inlined and compiled in release!
    
    // You can't code in unsafe Rust like it's C++.
    // Unsafe Rust does not allow you to jump over the borrow rules.
    // It merely stops preventing you from trying.
}

// #[inline(never)]
pub fn foo(x: &mut i32, y: &mut i32) -> i32 {
    *x = 42;
    *y = 0;
    *x
}